chmod
===

用来变更文件或目录的权限

## 补充说明

**chmod命令** 用来变更文件或目录的权限。在UNIX系统家族里，文件或目录权限的控制分别以读取、写入、执行3种一般权限来区分，另有3种特殊权限可供运用。用户可以使用chmod指令去变更文件与目录的权限，设置方式采用文字或数字代号皆可。符号连接的权限无法变更，如果用户对符号连接修改权限，其改变会作用在被连接的原始文件。

权限范围的表示法如下：

`u` User，即文件或目录的拥有者；  
`g` Group，即文件或目录的所属群组；  
`o` Other，除了文件或目录拥有者或所属群组之外，其他用户皆属于这个范围；  
`a` All，即全部的用户，包含拥有者，所属群组以及其他用户；  
`r` 读取权限，数字代号为“4”;  
`w` 写入权限，数字代号为“2”；  
`x` 执行或切换权限，数字代号为“1”；  
`-` 不具任何权限，数字代号为“0”；  
`s` 特殊功能说明：变更文件或目录的权限。

### 语法  

```
chmod(选项)(参数)
```

### 选项  

```bash
u  # 操作对象简称，用户user，文件或目录的所有者。
g  # 操作对象简称，同组用户group，文件或目录所属群组
o  # 操作对象简称，其它用户others
a  # 操作对象简称，所有用户all，系统默认使用此项
+  # 权限操作符，添加某些权限
-  # 权限操作符，取消某些权限
=  # 权限操作符，设置文件的权限为给定的权限
r  # 权限设定（英文），表示可读权限
w  # 权限设定（英文），表示可写权限
x  # 权限设定（英文），表示可执行权限
-  # 权限设定（英文字符），表示没有权限
X  # 权限设定，如果目标文件是可执行文件或目录，可给其设置可执行权限
s  # 权限设定，设置权限suid和sgid，使用权限组合“u+s”设定文件的用户的ID位，“g+s”设置组ID位
t  # 权限设定，只有目录或文件的所有者才可以删除目录下的文件
-c或——changes    # 效果类似“-v”参数，但仅回报更改的部分，如果文件权限已经改变，显示其操作信息；
-f或--quiet或——silent # 操作过程中不显示任何错误信息；
-R或——recursive  # 递归处理，将指令目录下的所有文件及子目录一并处理；
-v或——verbose    # 显示命令运行时的详细执行过程；
--reference=<参考文件或目录> # 把指定文件或目录的所属群组全部设成和参考文件或目录的所属群组相同；
<权限范围>+<权限设置> # 开启权限范围的文件或目录的该选项权限设置；
<权限范围>-<权限设置> # 关闭权限范围的文件或目录的该选项权限设置；
<权限范围>=<权限设置> # 指定权限范围的文件或目录的该选项权限设置；
--help    # 显示帮助信息
--version # 显示版本信息
```

### 参数  

权限模式：指定文件的权限模式；  
文件：要改变权限的文件。

### 知识扩展和实例  

Linux用 户分为：拥有者、组群(Group)、其他（other），Linux系统中，预设的情況下，系统中所有的帐号与一般身份使用者，以及root的相关信 息， 都是记录在`/etc/passwd`文件中。每个人的密码则是记录在`/etc/shadow`文件下。 此外，所有的组群名称记录在`/etc/group`內！

linux文件的用户权限的分析图

```bash
  -rw-r--r--   1 user  staff   651 Oct 12 12:53 .gitmodules
# ↑╰┬╯╰┬╯╰┬╯
# ┆ ┆  ┆  ╰┈ 0 其他人
# ┆ ┆  ╰┈┈┈┈┈┈┈┈┈┈┈┈┈┈┈ g 属组
# ┆ ╰┈┈┈┈ u 属组
# ╰┈┈ 第一个字母 `d` 代表目录，`-` 代表普通文件
```

例：rwx　rw-　r--

r=读取属性　　//值＝4  
w=写入属性　　//值＝2  
x=执行属性　　//值＝1

```bash
chmod u+x,g+w f01　　# 为文件f01设置自己可以执行，组员可以写入的权限
chmod u=rwx,g=rw,o=r f01
chmod 764 f01
chmod a+x f01　　    # 对文件f01的u,g,o都设置可执行属性
```

文件的属主和属组属性设置

```bash
chown user:market f01　　# 把文件f01给uesr，添加到market组
ll -d f1  查看目录f1的属性
```


将`/home/wwwroot/`里的所有文件和文件夹设置为`755`权限

(1)直接指定路径修改

```bash
chmod -R  755 /home/wwwroot/*
```

(2)手动进入该目录修改权限（并显示详细过程）


```bash
cd /home/wwwroot
chmod -Rv  755 *       #注意：“*”表示通配符，指的是所有文件和文件
sudo chmod o+rw /shared #设置文件夹权限以保证其余用户可以访问它。
```


<!-- Linux命令行搜索引擎：https://jaywcjlove.github.io/linux-command/ -->
